{-# LANGUAGE DeriveGeneric #-}
module Library.Types.Player where

import Data.Aeson

import GHC.Generics (Generic)

import Linear

-- | Data object for storing player state
data Wizard = Wizard
  { wizardPos    :: Position
    -- ^ Player's position
  , wizardRot    :: Float
    -- ^ Player's view direction
  , wizardHealth :: Int
    -- ^ Player's hit points
  , wizardMana   :: Int
    -- ^ Player's Mana (magic fuel)
  , wizardWands  :: [Wand]
    -- ^ Player's wands (read: weapons)
  , wizardEffects :: [Effect]
    -- ^ 'Effect's affecting the player
  }
  deriving (Show, Eq, Generic)

instance FromJSON Wizard

instance ToJSON Wizard

-- | Just a type synonym for any kind of position. Translates to a 2D vector
type Position = V2 Float

instance FromJSON Position

instance ToJSON Position

-- List of all available Wands
data Wand
  = HealthUp  -- ^ Instant Health boost
  | ManaUp    -- ^ Instant Mana boost
  | Shield    -- ^ Magical shield protecting the player
  | BigWand   -- ^ Stronger Zap spell
  | FireWand  -- ^ Spell for causing fire damage
  | IceWand   -- ^ Spell for causing ice damage
  | BlinkWand -- ^ Spell for random teleport
  deriving (Show, Eq, Enum, Bounded, Generic)

instance FromJSON Wand

instance ToJSON Wand

data Effect = Effect
  { effectType :: Affliction -- ^ What is happening
  , effectTTL  :: Float      -- ^ remaining time
  }
  deriving (Show, Eq, Generic)

instance FromJSON Effect

instance ToJSON Effect

data Affliction
  = Burning  -- ^ Damage over time
  | Frozen   -- ^ Unable to perform any action
  | Shielded -- ^ invulnerable until next hit or time runs out.
  deriving (Show, Eq, Generic)

instance FromJSON Affliction

instance ToJSON Affliction

harmingAfflictions :: [ Affliction ]
harmingAfflictions =
  [ Burning
  , Frozen
  ]
